/**
 *******************************************************************************
 * @file    adc.c
 * @brief   The application functions of ADC_Data_Read demo for the
 *          TOSHIBA 'TMPM330' Device Series 
 * @version V1.001
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */
/* Includes ------------------------------------------------------------------*/
#include "adc.h"

/**
  * @brief  Initialize ADC register and variables
  * @param  None
  * @retval None
  */

void ADC_Init()
{
    ADC_SWReset();
    ADC_SetInputChannel(ADC_AN_0);
    ADC_SetRepeatMode(ENABLE);
    ADC_SetINTMode(ADC_INT_SIGNLE);
    ADC_SetVref(ENABLE);
    Delay(10);                  /* wait at least 0.003ms to ensure the voltage is stable */
}

/**
  * @brief  Send ADC Date value Time value to LCD.
  * @param  None
  * @retval None
  */
void ADC_Display()
{
    uint32_t e_volumn;
    uint32_t e_ADC_i;
    char tADC_Display[16];
    ADC_ResultTypeDef ADC_Result;
    ADC_Result = ADC_GetConvertResult(ADC_REG_08);
    e_volumn = (ADC_Result.ADCResultValue * 100U) / 0x03ffU;    /* Max ADC is 0x03ff      */

    tADC_Display[0] = 'A';      /* Disaply: "ADC Value:   "     */
    tADC_Display[1] = 'D';
    tADC_Display[2] = 'C';
    tADC_Display[3] = ' ';
    tADC_Display[4] = 'V';
    tADC_Display[5] = 'a';
    tADC_Display[6] = 'l';
    tADC_Display[7] = 'u';
    tADC_Display[8] = 'e';
    tADC_Display[9] = ':';

    /*  Display the value                           */
    if (e_volumn >= 100U) {
        tADC_Display[10] = (char) ((e_volumn / 100U) + 0x30U);
        tADC_Display[11] = (char) (((e_volumn / 10U) % 10U) + 0x30U);
        tADC_Display[12] = (char) ((e_volumn % 10U) + 0x30U);
    } else {
        tADC_Display[10] = ' ';
        if (e_volumn / 10U) {
            tADC_Display[11] = (char) ((e_volumn / 10U) + 0x30U);
        } else {
            tADC_Display[11] = ' ';
        }
        tADC_Display[12] = (char) ((e_volumn % 10U) + 0x30U);
    }
    tADC_Display[13] = 0U;
    Send_To_LCD(FIRST_LINE, LCD_COMMAND);
    Send_LCD_Text(tADC_Display);

    /*  Display the value volumn percent, the 100% is 16 x '*'      */
    e_volumn = e_volumn * 16U;
    e_volumn = e_volumn / 100U;

    for (e_ADC_i = 0U; e_ADC_i < e_volumn; e_ADC_i++) {
        tADC_Display[e_ADC_i] = '*';    /* Display "*" for volumn percent     */
    }
    tADC_Display[e_ADC_i] = 0U; /* String end               */

    Send_To_LCD(SECOND_LINE, LCD_COMMAND);
    Send_LCD_Text(tADC_Display);
}
