/**
 *******************************************************************************
 * @file    flash.h
 * @brief   header file for flash.c, included some address definition and
 *          function prototypes
 * @version V1.001
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

#ifndef __FLASH_H
#define __FLASH_H

/* Includes ------------------------------------------------------------------*/
/* #define _BOOT_ROM */
#include "TMPM330.h"
#include "tmpm330_fc.h"

/* Private define ------------------------------------------------------------*/
#ifdef _BOOT_ROM
#define FLASH_BEGIN ((uint32_t)0x3f800000)
#else
#define FLASH_BEGIN ((uint32_t)0x00000000)
#endif

#define FLASH_END (FLASH_BEGIN+(uint32_t)0x0007ffff)
#define FLASH_PAGE_SIZE ((uint32_t)0x00000200)

#define FLASH_CMD_BC1_ADDR ((uint32_t)0x00005400)       /* address for bus cycle 1 of command, also used in other cycles */
#define FLASH_CMD_BC2_ADDR ((uint32_t)0x0000aa00)       /* address for bus cycle 2 of command, also used in other cycles */
#define FLASH_CMD_BC1_DATA ((uint32_t)0x000000aa)       /* data for bus cycle 1 of command, also used in other cycles */
#define FLASH_CMD_BC2_DATA ((uint32_t)0x00000055)       /* data for bus cycle 2 of command, also used in other cycles */

#define OVER_TIME   0xffffU

/* error code */
#define ERR_NONE    0x0U        /* no error */
#define ERR_FA      0x81U       /* address is not in the falsh space */
#define ERR_PRO     0x82U       /* block is protected */
#define ERR_OT      0x83U       /* overtime */
#define ERR_SIZE    0x84U       /* error size */
#define ERR_INT_ACT 0x85U       /* try to enter low power mode when there are some low power mode clear factors active */
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Global variables ----------------------------------------------------------*/
/* external variables --------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/
uint8_t FLASH_EraseBlock(uint32_t * block_addr);
uint8_t Write_Flash(uint32_t * addr_dest, uint32_t * addr_source, uint32_t len);
uint8_t FLASH_WritePage(uint32_t * page_addr, uint32_t * data, uint16_t size);

#endif                          /* #ifndef __ERR_C_H */
/*************************** END OF FILE **************************************/
