/**
 *******************************************************************************
 * @file    main.c
 * @brief   the application functions of RMC Power_Mode_Change demo for the
 *          TOSHIBA 'TMPM330' Device Series
 * @version V1.001
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "main.h"

/* external variables --------------------------------------------------------*/
extern RMCSTAT fRMCDone;
extern RMCPRS fRMCPress;

/**
  * @brief  main  function
  * @param  None
  * @retval None
  */
int main(void)
{
    SYSSTAT fSysSleep;

    /* Initialize Variables */
    gSIORdIndex = 0U;
    gSIOWrIndex = 0U;
    fSIO0_INT = 0U;
    fTMRB0_INT = 0U;
    fSysSleep = WAKE;

    fRMCDone = RMC_NON;
    fRMCPress = RELEASE;

    SIO_Configuration(UART0);
    SIO_ChInit();

    Timer_ChInit(TSB_TB0);      /* configure TMRB0 */

    RMC_Configuration(TSB_RMC1);        /*RMC1 IO initial  */
    RMC_ChInit(TSB_RMC1);       /* configure RMC1  */

    /* set source(RMC1 interrupt) to exit sleep mode */
    CG_SetSTBYReleaseINTSrc(CG_INT_SRC_RMC_RX_1, CG_INT_ACTIVE_STATE_RISING, ENABLE);

    NVIC_EnableIRQ(INTTX0_IRQn);
    NVIC_EnableIRQ(INTRMCRX1_IRQn);

    printf("TMPM330 RMC Power Mode Change DEMO\r\n");
    for (;;) {
        if (fRMCDone == RMC_DONE) {     /* RMC reception complete */
            RMC_SetRxCtrl(TSB_RMC1, DISABLE);   /* stop RMC receiving */
            RMCRcv();           /* get TSB_RMC1 gAddress and gCommand */
            switch (gRMC_Command) {
                /* special keys: sleep -> normal */
            case SEVEN_KEY:
            case CH_UP:
            case VOL_UP:
                PrintCode(FORMAT, gRMC_Address, gRMC_Command);
                if (fSysSleep == SLEEP) {       /* current state is sleep */
                    fSysSleep = WAKE;
                    printf("Wakeup from Sleep\r\n");
                } else {
                    /* do nothing */
                }
                break;
                /* special keys: normal -> sleep */
            case NINE_KEY:
            case VOL_DOWN:
            case CH_DOWN:
                if (fSysSleep == WAKE) {        /* current state is normal */
                    fSysSleep = SLEEP;
                    PrintCode(FORMAT, gRMC_Address, gRMC_Command);
                    printf("Now, Going to Sleep\r\n");
                    RMC_SetRxCtrl(TSB_RMC1, ENABLE);    /* enable RMC receive */
                    Delay(250U);        /* wait for all interrupts over */
                    PowerMode_N2Sleep();        /* go to sleep */
                } else {
                    RMC_SetRxCtrl(TSB_RMC1, ENABLE);    /* enable RMC receive */
                    Delay(250U);        /* wait for all interrupts over */
                    PowerMode_N2Sleep();        /* go back to sleep */
                }
                break;
                /* other keys */
            default:
                if (fSysSleep == WAKE) {        /* current state is normal */
//                    PrintCode(FORMAT, gRMC_Address, gRMC_Command);
                } else {
                    RMC_SetRxCtrl(TSB_RMC1, ENABLE);    /* enable RMC receive */
                    Delay(250U);        /* wait for all interrupts over */
                    PowerMode_N2Sleep();        /* go back to sleep */
                }
                break;
            }
            RMC_SetRxCtrl(TSB_RMC1, ENABLE);    /* enable RMC receive */
        } else {
            /* do nothing */
        }
    }
}

/**
  * @brief  Print code received by RMC.
  * @param  Format name, address, command
  * @retval None
  */

void PrintCode(char *ch, uint16_t addr, uint32_t cmd)
{
#ifdef RMC_1
    printf("%s: %02x %02x\r\n", ch, addr, cmd);
#endif

#ifdef RMC_2
    printf("%s: %02x %02x\r\n", ch, addr, cmd);
#endif

#ifdef RMC_3
    printf("%s: %04x %06x\r\n", ch, addr, cmd);
#endif

#ifdef RMC_4
    if (gRMC_4Address != 13U) {
        printf("%s: %02x %02x\r\n", ch, addr, cmd);
    } else {
        printf("%s: %04x %02x\r\n", ch, addr, cmd);
    }
#endif

#ifdef RMC_5
    printf("%s: %02x %02x\r\n", ch, addr, cmd);
#endif
}

/**
  * @brief  Deal with the error parameter
  * @param  file: Pointer to the file where the error parameter locates
  * @param  line: Number of the line in which the error parameter locates
  * @retval None
  */

#ifdef DEBUG
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif
