/** 
 *******************************************************************************
 * @file    main.h
 * @brief   the application functions of RMC Power_Mode_Change demo for the TOSHIBA
 *          'TMPM330' Device Series 
 * @version V1.001
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */
/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __MAIN_H
#define __MAIN_H
/* Includes ------------------------------------------------------------------*/
#include "tmpm330_tmrb.h"
#include "tmpm330_uart.h"
#include "tmpm330_rmc.h"
#include "tmpm330_cg.h"
#include "tmpm330_gpio.h"
#include "tmpm330_int.h"
#include "uart.h"
#include "timer.h"
#include "rmc.h"
#include "power_mode.h"
#include "lcd.h"
#include <stdio.h>
/* external variables --------------------------------------------------------*/
extern uint8_t gSIORdIndex, gSIOWrIndex, fSIO0_INT;
extern uint8_t fTMRB0_INT;
extern uint32_t gRMC_Command;
extern uint16_t gRMC_Address;
#ifdef RMC_4
extern uint8_t gRMC_4Address;
#endif

/* Private define ------------------------------------------------------------*/
typedef enum { SLEEP = 0, WAKE = 1 } SYSSTAT;

#ifdef RMC_1
/************** FOR RMC_1 FORMAT ****************/
#define SEVEN_KEY   ((uint8_t)0xE0U)    /* KEY 7 code */
#define NINE_KEY    ((uint8_t)0x90U)    /* KEY 9 code */
#define VOL_UP      ((uint8_t)0x58U)    /* KEY Vol UP code */
#define VOL_DOWN    ((uint8_t)0x78U)    /* KEY Vol DOWN code */
#define CH_UP       ((uint8_t)0xd8U)    /* KEY CH UP code */
#define CH_DOWN     ((uint8_t)0xf8U)    /* KEY CH DOWN code */
#define FORMAT      "RMC_1"     /* FORMAT name */
#endif

#ifdef RMC_2
/************** FOR RMC_2 FORMAT ****************/
#define SEVEN_KEY   ((uint8_t)0x07U)    /* KEY 7 code */
#define NINE_KEY    ((uint8_t)0x09U)    /* KEY 9 code */
#define VOL_UP      ((uint8_t)0x10U)    /* KEY Vol UP code */
#define VOL_DOWN    ((uint8_t)0x11U)    /* KEY Vol DOWN code */
#define CH_UP       ((uint8_t)0x20U)    /* KEY CH UP code */
#define CH_DOWN     ((uint8_t)0x21U)    /* KEY CH DOWN code */
#define FORMAT      "RMC_2"     /* FORMAT name */
#endif

#ifdef RMC_3
/************** FOR RMC_3 FORMAT ****************/
#define SEVEN_KEY   ((uint32_t)0x006869U)       /* KEY 7 code */
#define NINE_KEY    ((uint32_t)0x001819U)       /* KEY 9 code */
#define VOL_UP      ((uint32_t)0x000405U)       /* KEY Vol UP code */
#define VOL_DOWN    ((uint32_t)0x008485U)       /* KEY Vol DOWN code */
#define CH_UP       ((uint32_t)0x002C2DU)       /* KEY CH UP code */
#define CH_DOWN     ((uint8_t)0x00ACADU)        /* KEY CH DOWN code */
#define FORMAT      "RMC_3"     /* FORMAT name */
#endif

#ifdef RMC_4
/************** FOR RMC_4 FORMAT ****************/
#define SEVEN_KEY   ((uint8_t)0x30U)    /* KEY 7 code */
#define NINE_KEY    ((uint8_t)0x08U)    /* KEY 9 code */
#define VOL_UP      ((uint8_t)0x24U)    /* KEY Vol UP code */
#define VOL_DOWN    ((uint8_t)0x64U)    /* KEY Vol DOWN code */
#define CH_UP       ((uint8_t)0x04U)    /* KEY CH UP code */
#define CH_DOWN     ((uint8_t)0x44U)    /* KEY CH DOWN code */
#define FORMAT      "RMC_4"     /* FORMAT name */
#endif

#ifdef RMC_5
/************** FOR RMC_2 FORMAT ****************/
#define SEVEN_KEY   ((uint8_t)0xE0U)    /* KEY 7 code */
#define NINE_KEY    ((uint8_t)0x90U)    /* KEY 9 code */
#define VOL_UP      ((uint8_t)0x28U)    /* KEY Vol UP code */
#define VOL_DOWN    ((uint8_t)0xA8U)    /* KEY Vol DOWN code */
#define CH_UP       ((uint8_t)0x88U)    /* KEY CH UP code */
#define CH_DOWN     ((uint8_t)0x48U)    /* KEY CH DOWN code */
#define FORMAT      "RMC_5"     /* FORMAT name */
#endif

/* Private function prototypes -----------------------------------------------*/
void PrintCode(char *, uint16_t, uint32_t);
#endif
