/**
 *******************************************************************************
 * @file    main.c
 * @brief   The application functions of SBI EEPROM demo for the 
 *          TOSHIBA 'TMPM330' Device Series 
 * @version V1.001
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "main.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
Result trx_result;
SBI_InitI2CTypeDef myI2C;

uint32_t gI2C_WCnt;
uint32_t gI2C_RCnt;
uint32_t gI2C_TxDataLen;
uint32_t gI2C_TxData[16] = { 0 };

uint32_t gI2C_RxDataLen;
char gI2C_RxData[16] = { 0 };
char gSBIRxBuffer[SBI_BUFFER_SIZE] = { 0 };     /* used for receive data */
char gSBITxBuffer[SBI_BUFFER_SIZE] = { 0 };     /* used for send data */

TRXSTAT fSIORxOK = READY;
/* Global variables ----------------------------------------------------------*/
/* external variables --------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/

/**
  * @brief  Main routine
  * @param  None
  * @retval None
  */
int main(void)
{
    uint8_t iCnt, jCnt;
    SBI_I2CState i2c_state;

    KEY_Configuration();
    LCD_Configuration();
    LCD_Light(ON);
    SBI2_IO_Configuration();
    UART0_IO_Configuration();
    SIO_ChInit();
/*------------------------- Initialize SBI channel 2 -------------------------*/
    myI2C.I2CSelfAddr = SELF_ADDR;
    myI2C.I2CDataLen = SBI_I2C_DATA_LEN_8;
    myI2C.I2CACKState = ENABLE;
    myI2C.I2CClkDiv = SBI_I2C_CLK_DIV_328;
    SBI_Enable(TSB_SBI2);
    SBI_SWReset(TSB_SBI2);
    SBI_InitI2C(TSB_SBI2, &myI2C);
    NVIC_EnableIRQ(INTSBI2_IRQn);

    printf("TMPM330 SBI DEMO\r\n");
    for (;;) {

        Send_To_LCD(FIRST_LINE, LCD_COMMAND);
        Send_LCD_Text("Press K1 to");
        Send_To_LCD(SECOND_LINE, LCD_COMMAND);
        Send_LCD_Text("Read EEPROM");
        Key_WaitRelease(KEY1);
        iCnt = 0U;
/*---------------------------- Read EEPROM 16 bytes ---------------------------*/
        for (iCnt = 0U; iCnt < (TWOKB); iCnt++) {
            ClearBuffer(gI2C_RxData, SBI_BUFFER_SIZE);
            do {
                i2c_state = SBI_GetI2CState(TSB_SBI2);
            } while (i2c_state.Bit.BusState);
            gI2C_WCnt = 0U;
            gI2C_RCnt = 0U;
            gI2C_TxData[0] = iCnt * SIZE;
            gI2C_TxDataLen = 1U;
            gI2C_RxDataLen = SIZE;
            SBI_SetSendData(TSB_SBI2, EEPROM_ADDR | SBI_I2C_SEND);
            SBI_GenerateI2CStart(TSB_SBI2);

            while (gI2C_WCnt <= gI2C_TxDataLen);
            Delay(125);

            do {
                i2c_state = SBI_GetI2CState(TSB_SBI2);
            } while (i2c_state.Bit.BusState);

            SBI_SetSendData(TSB_SBI2, EEPROM_ADDR | SBI_I2C_RECEIVE);
            SBI_GenerateI2CStart(TSB_SBI2);

            while (gI2C_RCnt <= gI2C_RxDataLen);
            Delay(125);
            do {
                i2c_state = SBI_GetI2CState(TSB_SBI2);
            } while (i2c_state.Bit.BusState);

            Send_To_LCD(FIRST_LINE, LCD_COMMAND);
            Send_LCD_Text("TMPM330");
            Send_To_LCD(SECOND_LINE, LCD_COMMAND);
            Send_LCD_Text("EEPROM READ");
            if (!i2c_state.Bit.BusState) {
                printf("0x%04x: ", iCnt * SIZE);
                for (jCnt = 0U; jCnt < SIZE; jCnt++) {
                    printf("0x%02x ", gI2C_RxData[jCnt]);
                }
                printf("\r\n");
            } else {
                Send_LCD_Text("Read Error");
                printf("READ ERROR: %d\r\n", iCnt);
            }

        }
        printf("Finish\r\n");

        Send_To_LCD(FIRST_LINE, LCD_COMMAND);
        Send_LCD_Text("Read Over Press");
        Send_To_LCD(SECOND_LINE, LCD_COMMAND);
        Send_LCD_Text("K2 to Write");
        Key_WaitRelease(KEY2);
        ClearBuffer(gSIORxBuffer, SBI_BUFFER_SIZE);

        NVIC_EnableIRQ(INTRX0_IRQn);
        NVIC_EnableIRQ(INTTX0_IRQn);
        Send_To_LCD(FIRST_LINE, LCD_COMMAND);
        Send_LCD_Text("  Type a word  ");
        Send_To_LCD(SECOND_LINE, LCD_COMMAND);
        Send_LCD_Text(" ");

        while (fSIORxOK == READY) {     /* wait for UART receive over */
            /* do nothing */
        }
        fSIORxOK = READY;
        printf("\nTMPM330 UART Rx OK!\r\n");

/*---------------------------- Write EEPROM n bytes --------------------------*/
        gI2C_TxData[0] = SUB_ADDR;
        for (jCnt = 0U; jCnt < strlen(gSIORxBuffer); jCnt++) {
            gI2C_TxData[jCnt + 1U] = gSIORxBuffer[jCnt];
        }
        gI2C_TxDataLen = jCnt + 1U;
        gI2C_RxDataLen = 0U;
        gI2C_WCnt = 0U;
        do {
            i2c_state = SBI_GetI2CState(TSB_SBI2);
        } while (i2c_state.Bit.BusState);
        SBI_SetSendData(TSB_SBI2, EEPROM_ADDR | SBI_I2C_SEND);
        SBI_GenerateI2CStart(TSB_SBI2);
        while (gI2C_WCnt <= gI2C_TxDataLen);
        Delay(125);

        i2c_state = SBI_GetI2CState(TSB_SBI2);
        if (i2c_state.Bit.BusState) {
            Send_To_LCD(FIRST_LINE, LCD_COMMAND);
            Send_LCD_Text("Write ERROR");
        } else {
            Send_To_LCD(FIRST_LINE, LCD_COMMAND);
            Send_LCD_Text("Write Over");
        }

        Send_To_LCD(SECOND_LINE, LCD_COMMAND);
        Send_LCD_Text("Press K1 to Read");

        Key_WaitRelease(KEY1);

/*---------------------------- Read EEPROM n bytes ---------------------------*/
        ClearBuffer(gI2C_RxData, SBI_BUFFER_SIZE);
        do {
            i2c_state = SBI_GetI2CState(TSB_SBI2);
        } while (i2c_state.Bit.BusState);
        gI2C_WCnt = 0U;
        gI2C_RCnt = 0U;
        gI2C_TxData[0] = SUB_ADDR;
        gI2C_TxDataLen = 1U;
        gI2C_RxDataLen = strlen(gSIORxBuffer);
        SBI_SetSendData(TSB_SBI2, EEPROM_ADDR | SBI_I2C_SEND);
        SBI_GenerateI2CStart(TSB_SBI2);

        while (gI2C_WCnt <= gI2C_TxDataLen);
        Delay(125);

        do {
            i2c_state = SBI_GetI2CState(TSB_SBI2);
        } while (i2c_state.Bit.BusState);

        SBI_SetSendData(TSB_SBI2, EEPROM_ADDR | SBI_I2C_RECEIVE);
        SBI_GenerateI2CStart(TSB_SBI2);

        while (gI2C_RCnt <= gI2C_RxDataLen);
        Delay(125);
        do {
            i2c_state = SBI_GetI2CState(TSB_SBI2);
        } while (i2c_state.Bit.BusState);

        Send_To_LCD(FIRST_LINE, LCD_COMMAND);
        if (gI2C_RxData[gI2C_RxDataLen] == gSIORxBuffer[gI2C_RxDataLen]) {
            Send_LCD_Text(gI2C_RxData);
        } else {
            Send_LCD_Text("Read Error");
        }

        Send_To_LCD(SECOND_LINE, LCD_COMMAND);
        Send_LCD_Text("Press K4 to Demo");
        Key_WaitRelease(KEY4);
    }
}

/**
  * @brief  Clean buffer
  * @param  buffer point, buffer length
  * @retval None
  */
void ClearBuffer(char *buf, uint8_t length)
{
    uint8_t cnt;

    for (cnt = 0U; cnt < length; cnt++) {
        buf[cnt] = 0U;
    }
}

/**
  * @brief  Config the GPIO for UART0
  * @param  None
  * @retval None
  */
void UART0_IO_Configuration(void)
{
    GPIO_EnableFuncReg(GPIO_PE, GPIO_FUNC_REG_1, GPIO_BIT_0);
    GPIO_EnableFuncReg(GPIO_PE, GPIO_FUNC_REG_1, GPIO_BIT_1);
    GPIO_SetOutputEnableReg(GPIO_PE, GPIO_BIT_0, ENABLE);
    GPIO_SetInputEnableReg(GPIO_PE, GPIO_BIT_1, ENABLE);
}

/**
  * @brief  Config the GPIO for SBI2
  * @param  None
  * @retval None
  */
void SBI2_IO_Configuration(void)
{
    GPIO_EnableFuncReg(GPIO_PG, GPIO_FUNC_REG_1, GPIO_BIT_4);
    GPIO_EnableFuncReg(GPIO_PG, GPIO_FUNC_REG_1, GPIO_BIT_5);
    GPIO_SetOutputEnableReg(GPIO_PG, GPIO_BIT_4, ENABLE);
    GPIO_SetOutputEnableReg(GPIO_PG, GPIO_BIT_5, ENABLE);
    GPIO_SetInputEnableReg(GPIO_PG, GPIO_BIT_4, ENABLE);
    GPIO_SetInputEnableReg(GPIO_PG, GPIO_BIT_5, ENABLE);
    GPIO_SetOpenDrain(GPIO_PG, GPIO_BIT_4, ENABLE);
    GPIO_SetOpenDrain(GPIO_PG, GPIO_BIT_5, ENABLE);
    /* Pull up for SDA&SCL */
    GPIO_SetPullUp(GPIO_PG, GPIO_BIT_5, ENABLE);
    GPIO_SetPullUp(GPIO_PG, GPIO_BIT_4, ENABLE);
}

/**
  * @brief  Key wait release
  * @param  None
  * @retval None
  */
void Key_WaitRelease(uint8_t key)
{
    switch (key) {
    case KEY1:
        while (TSB_PB_DATA_PB7);
        while (!TSB_PB_DATA_PB7);
        break;
    case KEY2:
        while (TSB_PB_DATA_PB6);
        while (!TSB_PB_DATA_PB6);
        break;
    case KEY3:
        while (TSB_PB_DATA_PB5);
        while (!TSB_PB_DATA_PB5);
        break;
    case KEY4:
        while (TSB_PB_DATA_PB4);
        while (!TSB_PB_DATA_PB4);
        break;
    default:
        break;
    }
}

#ifdef DEBUG
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif
