/**
 *******************************************************************************
 * @file    tmpm330_int.c
 * @brief   The ISRs of SBI EEPROM demo for the
 *          TOSHIBA 'TMPM330' Application Notes 
 * @version V1.001
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "tmpm330_int.h"
/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* external variables --------------------------------------------------------*/
extern TRXSTAT fSIORxOK;
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/**
  * @brief  Non-maskable Interrupt Service Routine
  * @param  None
  * @retval None
  */
void NMI_Handler(void)
{
    /* do nothing */
}

/**
  * @brief  Hard Fault Interrupt Service Routine
  * @param  None
  * @retval None
  */
void HardFault_Handler(void)
{
    /* do nothing */
}

/**
  * @brief  SVCall Interrupt Service Routine
  * @param  None
  * @retval None
  */
void SVC_Handler(void)
{
    /* do nothing */
}

/**
  * @brief  INTTX0 Interrupt Service Routine
  * @param  None
  * @retval fSIO0_INT
  */
void INTTX0_IRQHandler(void)
{
    if (gSIORdIndex < gSIOWrIndex) {    /* buffer is not empty */
        UART_SetTxData(UART0, gSIOTxBuffer[gSIORdIndex++]);     /* send data */
        fSIO0_INT = SET;        /* enable SIO0 int */
    } else {
        fSIO0_INT = CLEAR;      /* disable SIO0 int */
        NVIC_EnableIRQ(INTTX0_IRQn);
    }
    if (gSIORdIndex >= gSIOWrIndex) {   /* reset buffer index */
        gSIOWrIndex = CLEAR;
        gSIORdIndex = CLEAR;
    } else {
        /* do nothing */
    }
}

/**
  * @brief  SIO0 RX interrupt service routine
  * @param  None
  * @retval gSIORxBuffer
  */
void INTRX0_IRQHandler(void)
{
    static uint8_t rcvcount = CLEAR;
    uint8_t tmp = CLEAR;

    tmp = UART_GetRxData(UART0);
    if ((tmp == 0x0D) || (tmp == 0x0A)) {       /* receive enter */
        fSIORxOK = YES;         /* finish */
        rcvcount = CLEAR;       /* reset count */
        NVIC_EnableIRQ(INTRX0_IRQn);
    } else {
        if (rcvcount < LENGTH) {
            gSIORxBuffer[rcvcount++] = UART_GetRxData(UART0);
        } else {
            /* do nothing */
        }
    }
}

/**
  * @brief  Interrupt Service Routine for SBI2
  * @param  None
  * @retval None
  */
void INTSBI2_IRQHandler(void)
{
    uint32_t tmp = 0U;
    TSB_SBI_TypeDef *SBIx;
    SBI_I2CState sbi_sr;

    SBIx = TSB_SBI2;
    sbi_sr = SBI_GetI2CState(SBIx);

    if (sbi_sr.Bit.MasterSlave) {       /* Master mode     */
        if (sbi_sr.Bit.TRx) {   /* Tx mode        */
            if (sbi_sr.Bit.LastRxBit) { /* LRB=1: the receiver requires no further data. */
                SBI_GenerateI2CStop(SBIx);
            } else {            /* LRB=0: the receiver requires further data. */
                if (gI2C_WCnt < gI2C_TxDataLen) {
                    SBI_SetSendData(SBIx, gI2C_TxData[gI2C_WCnt]);      /* Send next data */
                } else if (gI2C_WCnt == gI2C_TxDataLen) {       /* I2C data send finished. */
                    SBI_GenerateI2CStop(SBIx);
                } else {
                    /* Do nothing */
                }
                gI2C_WCnt++;
            }
        } else {                /* Rx Mode */
            if (gI2C_RCnt > gI2C_RxDataLen) {
                SBI_GenerateI2CStop(SBIx);
                SBI_SetI2CACK(SBIx, ENABLE);
            } else {
                if (gI2C_RCnt == gI2C_RxDataLen) {      /* Rx last data */
                    SBI_SetI2CBitNum(SBIx, SBI_I2C_DATA_LEN_1);
                } else if (gI2C_RCnt == (gI2C_RxDataLen - 1U)) {        /* Rx the data second to last */
                    /* Not generate ACK for next data Rx end. */
                    SBI_SetI2CACK(SBIx, DISABLE);
                } else {
                    /* Do nothing */
                }
                tmp = SBI_GetReceiveData(SBIx);
                if (gI2C_RCnt > 0U) {
                    gI2C_RxData[gI2C_RCnt - 1U] = tmp;
                } else {
                    /* first read is dummy read */
                }
                gI2C_RCnt++;
            }
        }
    } else {                    /* Slave mode  */
        /* Do nothing */
    }
}

/*********************************** END OF FILE ******************************/
