/**
 *******************************************************************************
 * @file    uart.c
 * @brief   The UART functions of SBI EEPROM demo for the
 *          TOSHIBA 'TMPM330' Application Notes 
 * @version V1.001
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "uart.h"
#include <stdio.h>
/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
uint8_t gSIORdIndex = 0;
uint8_t gSIOWrIndex = 0;
uint8_t fSIO0_INT = 0;
TRXSTAT SIORxOK = READY;
char gSIORxBuffer[SIO_BUFFER_SIZE] = { 0 };
char gSIOTxBuffer[SIO_BUFFER_SIZE] = { 0 };

/* Global variables ----------------------------------------------------------*/
/* external variables --------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/**
  * @brief  SIO Configuration Initialize the uart port (SIO0).
  * @param  None
  * @retval None
  */
void SIO_ChInit(void)
{
    UART_InitTypeDef myUART;

    myUART.BaudRate = 115200U;
    myUART.DataBits = UART_DATA_BITS_8;
    myUART.StopBits = UART_STOP_BITS_1;
    myUART.Parity = UART_NO_PARITY;
    myUART.Mode = UART_ENABLE_RX | UART_ENABLE_TX;
    myUART.FlowCtrl = UART_NONE_FLOW_CTRL;

    UART_Enable(UART0);
    UART_Init(UART0, &myUART);
    NVIC_EnableIRQ(INTTX0_IRQn);
}

/**
  * @brief  Send character by SIO
  * @param  Character to be sent
  * @retval Character to be sent
  */

uint8_t send_char(uint8_t ch)
{
    while (gSIORdIndex != gSIOWrIndex) {        /* wait for finishing sending */
        /* do nothing */
    }
    gSIOTxBuffer[gSIOWrIndex++] = ch;   /* fill TxBuffer */
    if (fSIO0_INT != SET) {     /* if SIO0 INT disable, enable it */
        fSIO0_INT = SET;        /* set SIO0 INT flag */
        UART_SetTxData(UART0, gSIOTxBuffer[gSIORdIndex++]);
        NVIC_EnableIRQ(INTTX0_IRQn);
    }

    return ch;
}

/*********************************** END OF FILE ******************************/
