/** 
 *******************************************************************************
 * @file    main.c
 * @brief   the application functions of UART0_to_UART2 demo for the TOSHIBA
 *          'TMPM330' Device Series 
 * @version V1.001
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "main.h"

/* Private define ------------------------------------------------------------*/
#define BufSize (sizeof(TxBuffer) / sizeof(*(TxBuffer)))

uint8_t TxBuffer[] = "TMPM330";
uint8_t RxBuffer[BufSize] = { 0U };

const uint8_t NumToBeTx = BufSize - 1U;
uint8_t TxCounter = 0U;
uint8_t RxCounter = 0U;

int main(void)
{
    uint8_t key_info;
    uint8_t wait_key1 = 1U;

    Key_Configuration();
    LCD_Configuration();
    LCD_Light(ON);

    SIO_Configuration(UART0);
    SIO_Configuration(UART2);
    SIO_ChInit();

    NVIC_EnableIRQ(INTTX0_IRQn);
    NVIC_EnableIRQ(INTRX2_IRQn);

    Send_To_LCD(FIRST_LINE, LCD_COMMAND);
    Send_LCD_Text("UART0 to UART2");

    while (1) {

        Send_To_LCD(SECOND_LINE, LCD_COMMAND);
        Send_LCD_Text("Press K1: Start");

        while (wait_key1) {
            /* read keyboard input */
            key_info = GPIO_ReadData(KEY_PORT);

            if (key_info == K1) {
                /* wait for key1 released */
                do {
                    wait_key1 = 0U;
                    key_info = GPIO_ReadData(KEY_PORT) & KEYRELEASE;
                } while (key_info != KEYRELEASE);
            }
        }

        UART_SetTxData(UART0, (uint32_t) (TxBuffer[TxCounter++]));

        while (RxCounter < NumToBeTx) {
            /* Do nothing */
        }

        Send_To_LCD(SECOND_LINE, LCD_COMMAND);
        Send_LCD_Text((char *) RxBuffer);

        wait_key1 = 1U;

        while (wait_key1) {
            /* read keyboard input */
            key_info = GPIO_ReadData(KEY_PORT);

            if (key_info == K1) {
                /* wait for key1 released */
                do {
                    wait_key1 = 0U;
                    key_info = GPIO_ReadData(KEY_PORT) & KEYRELEASE;
                } while (key_info != KEYRELEASE);
            }
        }
        wait_key1 = 1U;
        ResetIdx();

    }
}

/**
  * @brief  Clear Buffer&Counter .
  * @param  None
  * @retval None
  */
void ResetIdx(void)
{
    uint8_t i = 0U;

    for (i = 0U; i <= RxCounter; i++) {
        RxBuffer[i] = 0U;
    }
    TxCounter = 0U;
    RxCounter = 0U;
}

/**
  * @brief  Key Configuration.
  * @param  None
  * @retval None
  */
void Key_Configuration(void)
{
    GPIO_SetPullUp(KEY_PORT, GPIO_BIT_4, ENABLE);
    GPIO_SetPullUp(KEY_PORT, GPIO_BIT_5, ENABLE);
    GPIO_SetPullUp(KEY_PORT, GPIO_BIT_6, ENABLE);
    GPIO_SetPullUp(KEY_PORT, GPIO_BIT_7, ENABLE);

    GPIO_SetInputEnableReg(KEY_PORT, GPIO_BIT_4, ENABLE);
    GPIO_SetInputEnableReg(KEY_PORT, GPIO_BIT_5, ENABLE);
    GPIO_SetInputEnableReg(KEY_PORT, GPIO_BIT_6, ENABLE);
    GPIO_SetInputEnableReg(KEY_PORT, GPIO_BIT_7, ENABLE);
}

/**
  * @brief  Deal with the error parameter
  * @param  file: Pointer to the file where the error parameter locates
  * @param  line: Number of the line in which the error parameter locates
  * @retval None
  */

#ifdef DEBUG
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif
