/** 
 *******************************************************************************
 * @file    uart.c
 * @brief   the application functions of UART0_to_UART2 demo for the TOSHIBA
 *          'TMPM330' Device Series 
 * @version V1.001
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "uart.h"

/**
  * @brief  SIO Configuration.
  * @param  None
  * @retval None
  */

void SIO_Configuration(TSB_SC_TypeDef * SCx)
{
    if (SCx == UART0) {
        GPIO_SetOutputEnableReg(GPIO_PE, GPIO_BIT_0, ENABLE);
        GPIO_EnableFuncReg(GPIO_PE, GPIO_FUNC_REG_1, GPIO_BIT_0);
        GPIO_EnableFuncReg(GPIO_PE, GPIO_FUNC_REG_1, GPIO_BIT_1);
        GPIO_SetInputEnableReg(GPIO_PE, GPIO_BIT_1, ENABLE);
    } else if (SCx == UART1) {
        GPIO_SetOutputEnableReg(GPIO_PE, GPIO_BIT_4, ENABLE);
        GPIO_EnableFuncReg(GPIO_PE, GPIO_FUNC_REG_1, GPIO_BIT_4);;
        GPIO_EnableFuncReg(GPIO_PE, GPIO_FUNC_REG_1, GPIO_BIT_5);;
        GPIO_SetInputEnableReg(GPIO_PE, GPIO_BIT_5, ENABLE);
    } else if (SCx == UART2) {
        GPIO_SetOutputEnableReg(GPIO_PF, GPIO_BIT_0, ENABLE);
        GPIO_EnableFuncReg(GPIO_PF, GPIO_FUNC_REG_1, GPIO_BIT_0);
        GPIO_EnableFuncReg(GPIO_PF, GPIO_FUNC_REG_1, GPIO_BIT_1);
        GPIO_SetInputEnableReg(GPIO_PF, GPIO_BIT_1, ENABLE);
    } else {
        /* Do nothing */
    }
}

/**
  * @brief  SIO Configuration Initialize the uart port (SIO0/SIO2).
  * @param  None
  * @retval None
  */

void SIO_ChInit(void)
{
    UART_InitTypeDef myUART;

    /* configure SIO0/SIO2 for reception */
    myUART.BaudRate = 115200U;  /* baut rate = 115200 */
    myUART.DataBits = UART_DATA_BITS_8; /* no handshake, 8-bit data, clock by baud rate generator */
    myUART.StopBits = UART_STOP_BITS_1; /* 1-bit stop, LSB,  W-buff enable */
    myUART.Parity = UART_NO_PARITY;
    myUART.Mode = UART_ENABLE_RX | UART_ENABLE_TX;
    myUART.FlowCtrl = UART_NONE_FLOW_CTRL;

    UART_Enable(UART0);
    UART_Init(UART0, &myUART);

    UART_Enable(UART2);
    UART_Init(UART2, &myUART);
}
