/** 
 *******************************************************************************
 * @file    main.c
 * @brief   the application functions of UART_Retarget demo for the TOSHIBA
 *          'TMPM330' Device Series 
 * @version V1.001
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "main.h"

/**
  * @brief  main  function
  * @param  None
  * @retval None
  */
int main(void)
{
    uint8_t key_info;
    uint8_t wait_key1 = 1U;
    char TxBuffer[] = "TMPM330";

    /* Initialize system */
    Key_Configuration();
    LCD_Configuration();
    LCD_Light(ON);
    SIO_Configuration(UART0);
    SIO_ChInit();

    /* enable SIO0 trx int */
    NVIC_EnableIRQ(INTTX0_IRQn);

    Send_To_LCD(FIRST_LINE, LCD_COMMAND);
    Send_LCD_Text("Retarget demo");

    while (1) {

        Send_To_LCD(SECOND_LINE, LCD_COMMAND);
        Send_LCD_Text("Press K1: Start");

        while (wait_key1) {
            /* read keyboard input */
            key_info = GPIO_ReadData(KEY_PORT);

            if (key_info == K1) {
                /* wait for key1 released */
                do {
                    wait_key1 = 0U;
                    key_info = GPIO_ReadData(KEY_PORT) & KEYRELEASE;
                } while (key_info != KEYRELEASE);
            }
        }

        printf("%s\r\n", TxBuffer);     /* SIO0 send data */

        while (fSIOTxOK == NO) {        /* wait for UART0 finish snd */
            /* do nothing */
        }
        fSIOTxOK = NO;

        Send_To_LCD(SECOND_LINE, LCD_COMMAND);
        Send_LCD_Text(TxBuffer);

        wait_key1 = 1U;

        while (wait_key1) {
            /* read keyboard input */
            key_info = GPIO_ReadData(KEY_PORT);

            if (key_info == K1) {
                /* wait for key1 released */
                do {
                    wait_key1 = 0U;
                    key_info = GPIO_ReadData(KEY_PORT) & KEYRELEASE;
                } while (key_info != KEYRELEASE);
            }
        }
        wait_key1 = 1U;
    }
}

/**
  * @brief  Key Configuration.
  * @param  None
  * @retval None
  */
void Key_Configuration(void)
{
    GPIO_SetPullUp(KEY_PORT, GPIO_BIT_4, ENABLE);
    GPIO_SetPullUp(KEY_PORT, GPIO_BIT_5, ENABLE);
    GPIO_SetPullUp(KEY_PORT, GPIO_BIT_6, ENABLE);
    GPIO_SetPullUp(KEY_PORT, GPIO_BIT_7, ENABLE);
    GPIO_SetInputEnableReg(KEY_PORT, GPIO_BIT_4, ENABLE);
    GPIO_SetInputEnableReg(KEY_PORT, GPIO_BIT_5, ENABLE);
    GPIO_SetInputEnableReg(KEY_PORT, GPIO_BIT_6, ENABLE);
    GPIO_SetInputEnableReg(KEY_PORT, GPIO_BIT_7, ENABLE);
}

/**
  * @brief  Deal with the error parameter
  * @param  file: Pointer to the file where the error parameter locates
  * @param  line: Number of the line in which the error parameter locates
  * @retval None
  */
#ifdef DEBUG
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif
